"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports._exportDataTypeToTypescript = _exportDataTypeToTypescript;
const node_opcua_data_model_1 = require("node-opcua-data-model");
const node_opcua_types_1 = require("node-opcua-types");
const node_opcua_utils_1 = require("node-opcua-utils");
const node_opcua_variant_1 = require("node-opcua-variant");
const private_stuff_1 = require("./private-stuff");
const cache_1 = require("./private/cache");
const get_corresponding_data_type_1 = require("./private/get_corresponding_data_type");
const utils2_1 = require("./utils2");
// eslint-disable-next-line max-statements, complexity
async function _exportDataTypeToTypescript(session, nodeId, cache, f) {
    const importCollect = undefined;
    const referenceBasicType = (name) => {
        const t = { name, namespace: -1, module: "BasicType" };
        importCollect && importCollect(t);
        cache.ensureImported(t);
        return t.name;
    };
    f = f || new node_opcua_utils_1.LineFile();
    const importCollector = (i) => {
        cache.ensureImported(i);
    };
    const nodeClass = node_opcua_data_model_1.NodeClass.DataType;
    const description = await (0, private_stuff_1.getDescription)(session, nodeId);
    const definition = await (0, private_stuff_1.getDefinition)(session, nodeId);
    const browseName = await (0, private_stuff_1.getBrowseName)(session, nodeId);
    const isAbstract = await (0, private_stuff_1.getIsAbstract)(session, nodeId);
    const interfaceImport = (0, private_stuff_1.makeTypeNameNew)(nodeClass, definition, browseName);
    const interfaceName = interfaceImport.name;
    const superType = await (0, private_stuff_1.getSubtypeNodeId)(session, nodeId);
    const baseInterfaceImport = (0, private_stuff_1.makeTypeNameNew)(nodeClass, definition, superType.browseName);
    cache.ensureImported(baseInterfaceImport);
    const baseInterfaceName = baseInterfaceImport.name;
    //  f.write(superType.toString());
    f.write(`/**`);
    if (description.text) {
        f.write((0, utils2_1.toComment)(" * ", description.text || ""));
        f.write(` *`);
    }
    f.write(` * |           |${(0, utils2_1.f1)(" ")}|`);
    f.write(` * |-----------|${(0, utils2_1.f2)("-")}|`);
    f.write(` * | namespace |${(0, utils2_1.f1)(cache.namespace[nodeId.namespace].namespaceUri)}|`);
    f.write(` * | nodeClass |${(0, utils2_1.f1)(node_opcua_data_model_1.NodeClass[nodeClass])}|`);
    f.write(` * | name      |${(0, utils2_1.f1)(browseName.name.toString())}|`);
    f.write(` * | isAbstract|${(0, utils2_1.f1)(isAbstract.toString())}|`);
    f.write(` */`);
    let type = "basic";
    if (definition instanceof node_opcua_types_1.EnumDefinition) {
        type = "enum";
        f.write(`export enum ${interfaceName}  {`);
        for (const field of definition.fields) {
            if (field.description.text) {
                f.write(`  /**`);
                f.write((0, utils2_1.toComment)("   * ", field.description.text || ""));
                f.write(`   */`);
            }
            f.write(`  ${(0, utils2_1.quotifyIfNecessary)(field.name)} = ${field.value[1]},`);
        }
        f.write(`}`);
    }
    else if (definition instanceof node_opcua_types_1.StructureDefinition) {
        if (baseInterfaceName === "DTUnion") {
            type = "structure";
            for (let i = 0; i < definition.fields.length; i++) {
                f.write(`export interface ${interfaceName}_${i} extends ${baseInterfaceName} {`);
                for (let j = 0; j < definition.fields.length; j++) {
                    const field = definition.fields[j];
                    const fieldName = (0, utils2_1.toJavascritPropertyName)(field.name, { ignoreConflictingName: false });
                    if (j === i) {
                        await outputStructureField(f, field);
                    }
                    else {
                        f.write(`  ${(0, utils2_1.quotifyIfNecessary)(fieldName)}?: never`);
                    }
                }
                f.write(`}`);
            }
            f.write(`export type ${interfaceName} = `);
            for (let i = 0; i < definition.fields.length; i++) {
                f.write(`  | ${interfaceName}_${i}`);
            }
            f.write(`  ;`);
        }
        else {
            type = "structure";
            if (!definition.fields.length && interfaceName !== "DTStructure") {
                f.write(`export type ${interfaceName} = ${baseInterfaceName};`);
            }
            else {
                if (interfaceName === "DTStructure") {
                    f.write(`export interface ${interfaceName} {`);
                }
                else {
                    f.write(`export interface ${interfaceName} extends ${baseInterfaceName} {`);
                }
                for (const field of definition.fields) {
                    await outputStructureField(f, field);
                }
                f.write(`}`);
                //
                referenceBasicType("ExtensionObject");
                // ache.ensureImported({ module: "BasicType", name: "ExtensionObject", namespace: 0 });
                // interface TighteningResultOptions extends Partial<DTTighteningResult> {}
                // interface TighteningResult extends ExtensionObject, DTTighteningResult {}
                const full = (0, cache_1.makeName2)(interfaceName);
                // filter issue with UDTOpticalVerifierScanResult that has a decode:Uint32 conflicting with the ExtensionObject decode method
                const toAvoid = ["decode", "encode"];
                const collidingNames = definition
                    .fields.map((a) => (0, utils2_1.toJavascritPropertyName)(a.name, { ignoreConflictingName: false }))
                    .filter((d) => toAvoid.indexOf(d.toLowerCase()) !== -1);
                const adpatedIntefaceName = collidingNames.length === 0
                    ? interfaceName
                    : `Omit<${interfaceName},${collidingNames.map((a) => `"${a}"`).join(",")}>`;
                f.write(`export interface ${full} extends ExtensionObject, ${adpatedIntefaceName} {};`);
            }
        }
    }
    else {
        type = "basic";
        f.write(`// NO DEFINITION`);
        f.write(`export type ${interfaceName} = ${baseInterfaceName};`);
        // throw new Error("Invalid " + definition?.constructor.name);
    }
    return { type, content: f.toString(), typeName: interfaceName };
    async function outputStructureField(f, field) {
        const fieldName = (0, utils2_1.toJavascritPropertyName)(field.name, { ignoreConflictingName: false });
        // special case ! fieldName=
        if (field.description.text) {
            f.write(`  /** ${field.description.text}*/`);
        }
        const opt = field.isOptional ? "?" : "";
        const arrayMarker = field.valueRank >= 1 ? "[]" : "";
        const { dataType, jtype } = await (0, get_corresponding_data_type_1.getCorrespondingJavascriptType)(session, field.dataType, cache, importCollector);
        f.write(`  ${(0, utils2_1.quotifyIfNecessary)(fieldName)}${opt}: ${jtype}${arrayMarker}; // ${node_opcua_variant_1.DataType[dataType]} ${field.dataType.toString()}`);
    }
}
//# sourceMappingURL=_dataType.js.map