import { LocalizedText, NodeClass, QualifiedName } from "node-opcua-data-model";
import { NodeId } from "node-opcua-nodeid";
import { IBasicSessionBrowseAsyncSimple, IBasicSessionReadAsyncSimple } from "node-opcua-pseudo-session";
import { ReferenceDescription } from "node-opcua-types";
import { LineFile } from "node-opcua-utils";
import { DataType } from "node-opcua-variant";
import { ModellingRuleType } from "node-opcua-address-space-base";
import { Cache, Import } from "./private/cache";
import { Options } from "./options";
export declare function convertDataTypeToTypescript(session: IBasicSessionReadAsyncSimple, dataTypeId: NodeId): Promise<void>;
interface ClassDefinition {
    nodeClass: NodeClass.VariableType | NodeClass.ObjectType;
    browseName: QualifiedName;
    isAbstract: boolean;
    description: LocalizedText;
    superType?: ReferenceDescription;
    baseClassDef?: ClassDefinition | null;
    children: ReferenceDescription[];
    members: ClassMember[];
    interfaceName: Import;
    baseInterfaceName: Import | null;
    dataTypeNodeId: NodeId | null;
    dataType: DataType;
    dataTypeName: string;
    dataTypeImport?: Import[];
    valueRank?: number;
    dataTypeCombination?: string;
}
export declare function makeTypeName2(nodeClass: NodeClass, browseName: QualifiedName, suffix?: string): Import;
export declare function extractClassDefinition(session: IBasicSessionReadAsyncSimple & IBasicSessionBrowseAsyncSimple, nodeId: NodeId, cache: Cache): Promise<ClassDefinition>;
interface ClassMemberBasic {
    name: string;
    childType: Import;
    isOptional: boolean;
    modellingRule: ModellingRuleType | null;
    description: LocalizedText;
    suffix?: string;
    suffixInstantiate?: string;
    chevrons: any;
    typeToReference: Import[];
}
interface ClassMember extends ClassMemberBasic {
    /**
     * the OPCUA name of the class member
     */
    browseName: QualifiedName;
    nodeClass: NodeClass.Object | NodeClass.Method | NodeClass.Variable;
    /**
     * the Typescript name of the class Member
     */
    name: string;
    modellingRule: ModellingRuleType | null;
    isOptional: boolean;
    /**
     * class Def
     */
    classDef: ClassDefinition | null;
    description: LocalizedText;
    typeDefinition: ReferenceDescription;
    childType: Import;
    children: ReferenceDescription[];
    children2: ClassMemberBasic[];
    dataTypeNodeId?: NodeId | null;
    dataType?: DataType;
    jtype?: string;
    suffix?: string;
    suffix2?: string;
    suffix3?: string;
    innerClass?: Import | null;
    dataTypeCombination?: string;
    childBase?: Import | null;
}
interface ClassifiedReferenceDescriptions {
    Mandatory: ReferenceDescription[];
    Optional: ReferenceDescription[];
    MandatoryPlaceholder: ReferenceDescription[];
    OptionalPlaceholder: ReferenceDescription[];
    ExposesItsArray: ReferenceDescription[];
}
export declare function findClassMember(session: IBasicSessionReadAsyncSimple & IBasicSessionBrowseAsyncSimple, browseName: QualifiedName, baseParentDef: ClassDefinition, cache: Cache): Promise<ClassMember | null>;
export declare function checkIfShouldExposeInnerDefinition(main: ClassifiedReferenceDescriptions, instance: ClassifiedReferenceDescriptions): boolean;
interface ClassDefinitionB {
    superType?: {
        nodeId: NodeId;
    };
    interfaceName: Import;
    baseClassDef?: ClassDefinition | null;
}
export declare function extractClassMemberDef(session: IBasicSessionReadAsyncSimple & IBasicSessionBrowseAsyncSimple, nodeId: NodeId, parentDef: ClassDefinitionB, cache: Cache): Promise<ClassMember>;
export declare function findUsedImport(namespaceIndex: number, cache: Cache): string[];
export type Type = "enum" | "basic" | "structure" | "ua";
/**
 *  nodeId : a DataType, ReferenceType,AObjectType, VariableType node
 */
export declare function _convertTypeToTypescript(session: IBasicSessionReadAsyncSimple & IBasicSessionBrowseAsyncSimple, nodeId: NodeId, cache: Cache, f?: LineFile): Promise<{
    type: Type;
    content: string;
    typeName: string;
}>;
export declare function convertTypeToTypescript(session: IBasicSessionReadAsyncSimple & IBasicSessionBrowseAsyncSimple, nodeId: NodeId, options: Options, cache?: Cache, f?: LineFile): Promise<{
    type: Type;
    content: string;
    module: string;
    folder: string;
    filename: string;
    dependencies: string[];
}>;
export {};
