"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const node_opcua_address_space_1 = require("node-opcua-address-space");
const nodeJS_1 = require("node-opcua-address-space/nodeJS");
const node_opcua_nodesets_1 = require("node-opcua-nodesets");
const convert_namespace_to_typescript_1 = require("./convert_namespace_to_typescript");
const update_parent_tsconfig_1 = require("./update_parent_tsconfig");
async function main() {
    const addressSpace = node_opcua_address_space_1.AddressSpace.create();
    const xmlFiles = node_opcua_nodesets_1.nodesetCatalog.map((set) => (0, node_opcua_nodesets_1.constructNodesetFilename)(set.xmlFile));
    await (0, nodeJS_1.generateAddressSpace)(addressSpace, xmlFiles);
    const session = new node_opcua_address_space_1.PseudoSession(addressSpace);
    const options = {
        baseFolder: path_1.default.join(__dirname, "../../"),
        prefix: "node-opcua-nodeset-",
    };
    // const nodesetCatalog2 = nodesetCatalog.filter((meta) => meta.name == "scales");
    await Promise.all(node_opcua_nodesets_1.nodesetCatalog.map((meta) => {
        const index = addressSpace.getNamespaceIndex(meta.uri);
        if (index === -1) {
            console.log("namespace not found", meta.uri);
            return;
        }
        return (0, convert_namespace_to_typescript_1.convertNamespaceTypeToTypescript)(session, index, {
            ...options,
            nsName: meta.name,
        });
    }));
    await (0, update_parent_tsconfig_1.updateParentTSConfig)();
}
void main();
//# sourceMappingURL=main.js.map