"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cache2 = exports.Cache = void 0;
exports.makeName2 = makeName2;
exports.referenceExtensionObject = referenceExtensionObject;
exports.referenceEnumeration = referenceEnumeration;
exports.constructCache = constructCache;
exports.makeTypeNameNew = makeTypeNameNew;
const path_1 = __importDefault(require("path"));
const node_opcua_assert_1 = __importDefault(require("node-opcua-assert"));
const node_opcua_data_model_1 = require("node-opcua-data-model");
const node_opcua_nodeid_1 = require("node-opcua-nodeid");
const node_opcua_types_1 = require("node-opcua-types");
const case_anything_1 = require("case-anything");
const utils_1 = require("./utils");
function makeName2(name) {
    return "U" + name;
    // if (name === "EUInformation") {
    //     return "EUInformation2";
    // }
    // return name.replace(/^DT/, "").replace(/^3/, "Three").replace(/^2/, "Two");
}
class Cache {
    constructor() {
        this.namespace = [];
        this.requestedSymbols = { namespace: [] };
        this.requestedBasicTypes = {};
        this.imports = {};
        this.imports = {
            "node-opcua-address-space-base": {
                UAObject: 1,
                UAVariable: 1,
                UAVariableT: 1,
                UAMethod: 1,
                UADataType: 1,
                UAProperty: 1
            },
            "node-opcua-variant": {
                DataType: 1,
                Variant: 1,
                VariantOptions: 1,
            },
            "node-opcua-data-model": {
                LocalizedText: 1,
                QualifiedName: 1,
                DiagnosticInfo: 1
            },
            "node-opcua-data-access": {
                EUInformation: 1
            },
            "node-opcua-nodeid": {
                NodeId: 1,
                ExpandedNodeId: 1
            },
            "node-opcua-status-code": {
                StatusCode: 1,
                StatusCodes: 1
            },
            "node-opcua-basic-types": {
                Int64: 1,
                UInt64: 1,
                UInt32: 1,
                Int32: 1,
                UInt16: 1,
                Int16: 1,
                Byte: 1,
                SByte: 1,
                UAString: 1,
                DateTime: 1,
                Guid: 1
            },
            "node-opcua-data-value": {
                DataValue: 1
            },
            "node-opcua-types": {
                EnumValueType: 1
            },
            "node-opcua-extension-object": {
                ExtensionObject: 1
            }
            // "node-opcua-schemas": {
            //     ExtensionObject: 1
            // }
        };
    }
    resetRequire() {
        this.requestedSymbols = { namespace: [] };
        this.requestedBasicTypes = {};
    }
    referenceBasicType(basicType) {
        this.requestedBasicTypes[basicType] = (this.requestedBasicTypes[basicType] || 0) + 1;
        return basicType;
    }
    _reference(namespaceIndex, typeName, mainTypeName) {
        if (namespaceIndex <= 0 && this.imports["node-opcua-address-space-base"][typeName]) {
            this.referenceBasicType(typeName);
            return;
        }
        {
            const sourceFolder = this.namespace[namespaceIndex]?.sourceFolder;
            if (!sourceFolder) {
                throw new Error("Cannot find namespace  " + namespaceIndex);
            }
        }
        const symbols = (this.namespace[namespaceIndex].symbols = this.namespace[namespaceIndex].symbols || {});
        symbols[typeName] = (symbols[typeName] || 0) + 1;
        const ns = this.requestedSymbols.namespace[namespaceIndex] || { symbols: {} };
        this.requestedSymbols.namespace[namespaceIndex] = ns;
        ns.symbols = ns.symbols || {};
        if (!mainTypeName) {
            mainTypeName = typeName;
        }
        const subs = (ns.symbols[mainTypeName] = ns.symbols[mainTypeName] || { subSymbols: {} });
        subs.subSymbols[typeName] = (subs.subSymbols[typeName] || 0) + 1;
    }
    ensureImported(i) {
        if (i.namespace < 0) {
            this.referenceBasicType(i.name);
        }
        else {
            this._reference(i.namespace, i.name, i.module);
        }
    }
}
exports.Cache = Cache;
function getSourceFolderForNamespace(browseName, options, cache) {
    const ns = browseName.namespaceIndex;
    const n = cache.namespaceArray[ns].split("/").filter((s) => s.length !== 0);
    const nn = (n[n.length - 1] !== "UA" && n[n.length - 2] !== "UA" ? n[n.length - 2] : "") + n[n.length - 1];
    const module = options.prefix + (0, case_anything_1.kebabCase)(nn);
    const namespaceFolder = path_1.default.join(options.baseFolder, module);
    const sourceFolder = path_1.default.join(namespaceFolder);
    return { sourceFolder, module };
}
async function referenceExtensionObject(session, extensionObjectDataType) {
    const browseName = await (0, utils_1.getBrowseName)(session, extensionObjectDataType);
    const definition = await (0, utils_1.getDefinition)(session, extensionObjectDataType);
    const dataTypeNameImport = makeTypeNameNew(node_opcua_data_model_1.NodeClass.DataType, definition, browseName);
    return dataTypeNameImport;
}
async function referenceEnumeration(session, enumerationDataType) {
    const browseName = await (0, utils_1.getBrowseName)(session, enumerationDataType);
    const definition = await (0, utils_1.getDefinition)(session, enumerationDataType);
    const dataTypeNameImport = makeTypeNameNew(node_opcua_data_model_1.NodeClass.DataType, definition, browseName);
    return dataTypeNameImport;
}
class Cache2 extends Cache {
    constructor() {
        super(...arguments);
        this.namespaceArray = [];
    }
}
exports.Cache2 = Cache2;
async function constructCache(session, options) {
    const cache = new Cache2();
    cache.namespaceArray = cache.namespaceArray.length
        ? cache.namespaceArray
        : (await session.read({
            nodeId: (0, node_opcua_nodeid_1.resolveNodeId)("Server_NamespaceArray"),
            attributeId: node_opcua_data_model_1.AttributeIds.Value
        })).value.value;
    for (let namespaceIndex = 0; namespaceIndex < cache.namespaceArray.length; namespaceIndex++) {
        const namespaceUri = cache.namespaceArray[namespaceIndex];
        const { sourceFolder, module } = getSourceFolderForNamespace(new node_opcua_data_model_1.QualifiedName({ name: namespaceUri, namespaceIndex }), options, cache);
        cache.namespace.push({
            namespaceUri,
            sourceFolder,
            module,
            symbols: {}
        });
    }
    return cache;
}
function makeBasicTypeImport(name) {
    return { name, module: "BasicType", namespace: -1 };
}
function makeTypeNameNew(nodeClass, definition, browseName, suffix) {
    (0, node_opcua_assert_1.default)(browseName && !!browseName.name, "expecting a class name here");
    const typeName = browseName.name.toString();
    if (typeName === "EUInformation") {
        return makeBasicTypeImport("EUInformation");
    }
    else if (typeName === "Enumeration") {
        return makeBasicTypeImport("Enumeration");
    }
    else if (typeName === "BaseObjectType") {
        return makeBasicTypeImport("UAObject");
    }
    else if (typeName === "BaseVariableType") {
        return makeBasicTypeImport("UAVariableT");
    }
    else if (typeName === "BaseDataType") {
        return makeBasicTypeImport("UADataType");
    }
    else {
        let name = "";
        switch (nodeClass) {
            case node_opcua_data_model_1.NodeClass.Method:
                return makeBasicTypeImport("UAMethod");
            case node_opcua_data_model_1.NodeClass.Variable:
            case node_opcua_data_model_1.NodeClass.Object:
            case node_opcua_data_model_1.NodeClass.VariableType:
            case node_opcua_data_model_1.NodeClass.ObjectType:
                name = `UA${typeName.replace(/Type$/, "")}` + (suffix || "");
                break;
            case node_opcua_data_model_1.NodeClass.DataType:
                if (definition instanceof node_opcua_types_1.EnumDefinition) {
                    name = `Enum${typeName.replace(/(DataType|Enumeration|Type)$/, "")}`;
                }
                else {
                    name = `DT${typeName.replace(/(DataType|Enumeration|Type)$/, "")}`;
                }
                break;
        }
        if (name === "") {
            throw new Error("Internal Error " + node_opcua_data_model_1.NodeClass[nodeClass] + " " + typeName + "browseName = " + browseName.toString());
        }
        return { name, module: name, namespace: browseName.namespaceIndex };
    }
}
//# sourceMappingURL=cache.js.map