import { LocalizedText, NodeClass, QualifiedName } from "node-opcua-data-model";
import { NodeId } from "node-opcua-nodeid";
import { IBasicSessionBrowseAsyncSimple, IBasicSessionReadAsyncSimple } from "node-opcua-pseudo-session";
import { DataTypeDefinition, ReferenceDescription } from "node-opcua-types";
import { ModellingRuleType } from "node-opcua-address-space-base";
import { DataType } from "node-opcua-variant";
export declare function getDefinition(session: IBasicSessionReadAsyncSimple, nodeId: NodeId): Promise<DataTypeDefinition | null>;
export declare function getBrowseName(session: IBasicSessionReadAsyncSimple, nodeId: NodeId): Promise<QualifiedName>;
export declare function getDataTypeNodeId(session: IBasicSessionReadAsyncSimple, nodeId: NodeId): Promise<NodeId | null>;
export declare function getIsAbstract(session: IBasicSessionReadAsyncSimple, nodeId: NodeId): Promise<boolean>;
export declare function getDescription(session: IBasicSessionReadAsyncSimple, nodeId: NodeId): Promise<LocalizedText>;
export declare function getNodeClass(session: IBasicSessionReadAsyncSimple, nodeId: NodeId): Promise<NodeClass>;
export declare function getChildren(session: IBasicSessionBrowseAsyncSimple, nodeId: NodeId): Promise<ReferenceDescription[]>;
export declare function getFolderElements(session: IBasicSessionBrowseAsyncSimple, nodeId: NodeId): Promise<ReferenceDescription[]>;
export declare function getModellingRule(session: IBasicSessionBrowseAsyncSimple, nodeId: NodeId): Promise<ModellingRuleType | null>;
export declare function isExtensionObject(session: IBasicSessionBrowseAsyncSimple, nodeId: NodeId): Promise<boolean>;
export declare function isEnumeration(session: IBasicSessionBrowseAsyncSimple, nodeId: NodeId): Promise<boolean>;
export declare function extractBasicDataType(session: IBasicSessionBrowseAsyncSimple, dataTypeNodeId: NodeId): Promise<DataType>;
export declare function getSubtypeNodeIdIfAny(session: IBasicSessionBrowseAsyncSimple, nodeId: NodeId): Promise<ReferenceDescription | null>;
export declare function getSubtypeNodeId(session: IBasicSessionBrowseAsyncSimple, nodeId: NodeId): Promise<ReferenceDescription>;
export declare function getTypeDefOrBaseType(session: IBasicSessionBrowseAsyncSimple, nodeId: NodeId): Promise<ReferenceDescription>;
export declare function getTypeDefinition(session: IBasicSessionBrowseAsyncSimple, nodeId: NodeId): Promise<ReferenceDescription>;
interface DataTypeInfoBasic {
    type: "basic";
    dataType: DataType;
}
interface DataTypeInfoEnum {
    type: "enum";
    dataType: DataType.Int32;
    enumerationId: NodeId;
    enumerationType?: string;
}
interface DataTypeInfoGenericNumber {
    type: "genericNumber";
    dataTypeCombination: string;
}
type DataTypeInfo = DataTypeInfoBasic | DataTypeInfoEnum | DataTypeInfoGenericNumber;
export declare function _convertNodeIdToDataTypeAsync(session: IBasicSessionReadAsyncSimple & IBasicSessionBrowseAsyncSimple, dataTypeId: NodeId): Promise<DataTypeInfo>;
export declare function getChildrenOrFolderElements(session: IBasicSessionBrowseAsyncSimple, nodeId: NodeId): Promise<ReferenceDescription[]>;
export declare function getValueRank(session: IBasicSessionReadAsyncSimple, nodeId: NodeId): Promise<number>;
export {};
