"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cleanUpTypescriptModule = cleanUpTypescriptModule;
const ts_morph_1 = require("ts-morph");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const typescript_1 = __importDefault(require("typescript"));
const dryRun = false;
function fixFile(file) {
    // Remove unused imports
    file.getImportDeclarations().forEach((importDecl) => {
        const namedImports = importDecl.getNamedImports();
        const usedNamedImports = namedImports.filter((namedImport) => {
            const name = namedImport.getNameNode().getText();
            return file.getDescendantsOfKind(typescript_1.default.SyntaxKind.Identifier)
                .some(id => id.getText() === name && id !== namedImport.getNameNode());
        });
        // If none are used, remove whole import
        if (usedNamedImports.length === 0) {
            console.log("removing wholee import", importDecl.getText());
            importDecl.remove();
        }
        else if (usedNamedImports.length < namedImports.length) {
            // Remove only unused named imports
            namedImports.forEach(named => {
                if (!usedNamedImports.includes(named)) {
                    console.log("removing name in import ", named.getText());
                    named.remove();
                }
            });
        }
    });
    // Remove unused local variables (function-scope or top-level)
    file.getVariableStatements().forEach(variableStatement => {
        variableStatement.getDeclarations().forEach(declaration => {
            const name = declaration.getName();
            const references = declaration.findReferences();
            const isUsed = references.some(ref => {
                return ref.getReferences().some(r => !r.isDefinition());
            });
            if (!isUsed) {
                console.log("removing unused variable=", name);
                declaration.remove();
            }
        });
        try {
            // If no declarations left in this statement, remove the whole statement
            if (variableStatement.getDeclarations().length === 0) {
                console.log("removing variable statement:", variableStatement.getText());
                variableStatement.remove();
            }
        }
        catch (err) {
            console.log(err.message);
        }
    });
    // Save changes
    if (!dryRun) {
        file.saveSync();
    }
}
async function cleanUpTypescriptModule(moduleFolder) {
    const project = new ts_morph_1.Project({
        tsConfigFilePath: path_1.default.join(moduleFolder, "tsconfig.json"), // Optional but recommended
    });
    const sourceFolder = path_1.default.join(moduleFolder, "source");
    const dirFiles = await fs_1.default.promises.readdir(sourceFolder);
    for (let f of dirFiles) {
        console.log("processing file: ", f);
        const fullPath = path_1.default.join(sourceFolder, f);
        const sourceFile = project.getSourceFileOrThrow(fullPath);
        fixFile(sourceFile);
    }
}
//# sourceMappingURL=remove_unused.js.map