"use strict";
// Purpose of this script is to help ensure that "references" array in packages/tsconfig.json
// stays up-to-date with nodeset package list.
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateParentTSConfig = updateParentTSConfig;
const fs_1 = require("fs");
const { readFile, writeFile } = fs_1.promises;
const path_1 = __importDefault(require("path"));
// Uncomment the following to get __dirname equivalent when compiling to ESM
//import { fileURLToPath } from 'url';
//const __dirname = path.dirname(fileURLToPath(import.meta.url));
const node_opcua_nodesets_1 = require("node-opcua-nodesets");
async function updateParentTSConfig() {
    const packagesFolder = path_1.default.join(__dirname, '..', '..');
    const parentTSConfigFile = path_1.default.join(packagesFolder, 'tsconfig.json');
    console.log(`Updating parent tsconfig.json file: ${parentTSConfigFile}`);
    const content = await readFile(parentTSConfigFile, 'utf8');
    // Strip out comments to avoid JSON parsing error
    const contentWithoutComments = content.replace(/\/\*.+?\*\//g, '')
        .replace(/\/\/.+?$/g, '');
    // TODO: Preserve comments automatically
    let parentTSConfig = "";
    try {
        parentTSConfig = JSON.parse(contentWithoutComments);
    }
    catch (err) {
        console.log(`Error parsing JSON from ${parentTSConfigFile}:`, err);
        console.log('Content without comments:', contentWithoutComments);
        throw err;
    }
    const unlistedReferences = [];
    for (const meta of node_opcua_nodesets_1.nodesetCatalog) {
        const packageName = `node-opcua-nodeset-${meta.packageName}`;
        const nodesetPackageFolder = path_1.default.join(packagesFolder, packageName);
        if (!(0, fs_1.existsSync)(nodesetPackageFolder)) {
            console.log(`Ignoring ${meta.packageName} since package folder does not exist (${nodesetPackageFolder})`);
            continue;
        }
        if (!parentTSConfig.references.some((ref) => ref.path === packageName)) {
            console.log(`Found nodeset package that was not listed in tsconfig.json: ${packageName}`);
            unlistedReferences.push({ path: packageName });
        }
    }
    function compareByPath(a, b) {
        if (a.path < b.path) {
            return -1;
        }
        if (a.path > b.path) {
            return 1;
        }
        return 0;
    }
    if (unlistedReferences.length > 0) {
        parentTSConfig.references.push(...unlistedReferences);
        parentTSConfig.references.sort(compareByPath);
        // Maintain the compact style `{ "path": "node-opcua-X" }` in references section
        // Also add the comment about Istanbul back
        const newJSON = JSON.stringify(parentTSConfig, null, "    ")
            .replace('"removeComments": false', '"removeComments": false /* to prevent Istanbul ignore statements in comments from disappearing */')
            .replace(/{\s*"path":\s*"([^"]+)"\s*}/g, '{ "path": "$1" }')
            + '\n';
        //console.log('Writing updated tsconfig.json file:', newJSON);
        await writeFile(parentTSConfigFile, newJSON, 'utf8');
    }
}
//# sourceMappingURL=update_parent_tsconfig.js.map