"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.walkThroughReferenceTypes = walkThroughReferenceTypes;
exports.walkThroughDataTypes = walkThroughDataTypes;
exports.walkThroughInterfaceTypes = walkThroughInterfaceTypes;
exports.walkThroughObjectTypes = walkThroughObjectTypes;
exports.walkThroughVariableTypes = walkThroughVariableTypes;
const node_opcua_data_model_1 = require("node-opcua-data-model");
const node_opcua_nodeid_1 = require("node-opcua-nodeid");
const node_opcua_types_1 = require("node-opcua-types");
async function _walkThroughTypes(session, nodeId, visitor, level, parent) {
    level = level || 0;
    if (!parent) {
        const dataValues = await session.read([
            { nodeId, attributeId: node_opcua_data_model_1.AttributeIds.BrowseName },
            { nodeId, attributeId: node_opcua_data_model_1.AttributeIds.NodeClass }
        ]);
        const browseName = dataValues[0].value.value;
        const nodeClass = dataValues[1].value.value;
        parent = new node_opcua_types_1.ReferenceDescription({
            isForward: true,
            nodeId: nodeId,
            browseName,
            nodeClass,
            referenceTypeId: (0, node_opcua_nodeid_1.resolveNodeId)("HasSubtype")
            // typeDefinition,
        });
    }
    const browseResult = await session.browse({
        browseDirection: node_opcua_data_model_1.BrowseDirection.Forward,
        nodeId,
        includeSubtypes: true,
        referenceTypeId: (0, node_opcua_nodeid_1.resolveNodeId)("HasSubtype"),
        resultMask: 0xff
    });
    for (const reference of browseResult.references || []) {
        if (visitor) {
            const r = reference;
            r.parent = parent;
            await visitor.visit(r, level);
        }
        await _walkThroughTypes(session, reference.nodeId, visitor, level + 1, reference);
    }
}
async function walkThroughReferenceTypes(session, nodeVisitor) {
    const baseReferenceTypeNodeId = (0, node_opcua_nodeid_1.resolveNodeId)("References");
    await _walkThroughTypes(session, baseReferenceTypeNodeId, nodeVisitor);
}
async function walkThroughDataTypes(session, nodeVisitor) {
    const baseDataTypeTypeNodeId = (0, node_opcua_nodeid_1.resolveNodeId)("BaseDataType");
    await _walkThroughTypes(session, baseDataTypeTypeNodeId, nodeVisitor);
}
async function walkThroughInterfaceTypes(session, nodeVisitor) {
    const baseInterfaceTypeNodeId = (0, node_opcua_nodeid_1.resolveNodeId)("BaseInterfaceType");
    await _walkThroughTypes(session, baseInterfaceTypeNodeId, nodeVisitor);
}
async function walkThroughObjectTypes(session, nodeVisitor) {
    const baseObjectTypeNodeId = (0, node_opcua_nodeid_1.resolveNodeId)("BaseObjectType");
    await _walkThroughTypes(session, baseObjectTypeNodeId, nodeVisitor);
}
async function walkThroughVariableTypes(session, nodeVisitor) {
    const baseObjectTypeNodeId = (0, node_opcua_nodeid_1.resolveNodeId)("BaseVariableType");
    await _walkThroughTypes(session, baseObjectTypeNodeId, nodeVisitor);
}
//# sourceMappingURL=walk_through.js.map